@@ -35,6 +35,10 @@ class WFRouterTask : public WFGenericTask
 public:
 	RouteManager::RouteResult *get_result() { return &this->result; }
 
+public:
+	void set_state(int state) { this->state = state; }
+	void set_error(int error) { this->error = error; }
+
 protected:
 	RouteManager::RouteResult result;
 	std::function<void (WFRouterTask *)> callback;
@@ -116,6 +120,17 @@ public:
 	WFNSPolicy *get_policy(const char *name);
 	WFNSPolicy *del_policy(const char *name);
 
+public:
+	WFNSPolicy *get_default_policy() const
+	{
+		return this->default_policy;
+	}
+
+	void set_default_policy(WFNSPolicy *policy)
+	{
+		this->default_policy = policy;
+	}
+
 private:
 	WFNSPolicy *default_policy;
 	struct rb_root root;
@@ -131,6 +146,8 @@ public:
 		this->root.rb_node = NULL;
 		this->default_policy = default_policy;
 	}
+
+	virtual ~WFNameService();
 };
 
 #endif
