@@ -1701,23 +1701,62 @@ int KafkaMessage::encode_message(int api_type, struct iovec vectors[], int max)
 	return it->second(vectors, max);
 }
 
-static int kafka_get_api_version(const kafka_api_t *api, const KafkaConfig& conf,
-								 int api_type, int mvers, int message_version)
+static int kafka_api_get_max_ver(int api_type)
 {
-	int min_vers = 0;
+	switch (api_type)
+	{
+	case Kafka_Metadata:
+		return 4;
+	case Kafka_Produce:
+		return 7;
+	case Kafka_Fetch:
+		return 11;
+	case Kafka_FindCoordinator:
+		return 2;
+	case Kafka_JoinGroup:
+		return 5;
+	case Kafka_SyncGroup:
+		return 3;
+	case Kafka_Heartbeat:
+		return 3;
+	case Kafka_OffsetFetch:
+		return 1;
+	case Kafka_OffsetCommit:
+		return 7;
+	case Kafka_ListOffsets:
+		return 1;
+	case Kafka_LeaveGroup:
+		return 1;
+	case Kafka_ApiVersions:
+		return 0;
+	case Kafka_SaslHandshake:
+		return 1;
+	case Kafka_SaslAuthenticate:
+		return 0;
+	case Kafka_DescribeGroups:
+		return 0;
+	default:
+		return 0;
+	}
+}
+
+static int kafka_get_api_version(const kafka_api_t *api, const KafkaConfig& conf,
+								 int api_type, int max_ver, int message_version)
+{
+	int min_ver = 0;
 
 	if (api_type == Kafka_Produce)
 	{
 		if (message_version == 2)
-			min_vers = 3;
+			min_ver = 3;
 		else if (message_version == 1)
-			min_vers = 1;
+			min_ver = 1;
 
 		if (conf.get_compress_type() == Kafka_Zstd)
-			min_vers = 7;
+			min_ver = 7;
 	}
 
-	return kafka_broker_get_api_version(api, api_type, min_vers, mvers);
+	return kafka_broker_get_api_version(api, api_type, min_ver, max_ver);
 }
 
 int KafkaMessage::encode_head()
@@ -1726,6 +1765,8 @@ int KafkaMessage::encode_head()
 		this->api_version = 0;
 	else
 	{
+		int max_ver = kafka_api_get_max_ver(this->api_type);
+
 		if (this->api->features & KAFKA_FEATURE_MSGVER2)
 			this->message_version = 2;
 		else if (this->api->features & KAFKA_FEATURE_MSGVER1)
@@ -1745,10 +1786,8 @@ int KafkaMessage::encode_head()
 			this->config.set_compress_type(Kafka_NoCompress);
 		}
 
-		int mver = this->api_mver_map[this->api_type];
-
 		this->api_version = kafka_get_api_version(this->api, this->config,
-												  this->api_type, mver,
+												  this->api_type, max_ver,
 												  this->message_version);
 	}
 
@@ -2065,22 +2104,6 @@ KafkaRequest::KafkaRequest()
 	this->encode_func_map[Kafka_ApiVersions] = std::bind(&KafkaRequest::encode_apiversions, this, _1, _2);
 	this->encode_func_map[Kafka_SaslHandshake] = std::bind(&KafkaRequest::encode_saslhandshake, this, _1, _2);
 	this->encode_func_map[Kafka_SaslAuthenticate] = std::bind(&KafkaRequest::encode_saslauthenticate, this, _1, _2);
-
-	this->api_mver_map[Kafka_Metadata] = 4;
-	this->api_mver_map[Kafka_Produce] = 7;
-	this->api_mver_map[Kafka_Fetch] = 11;
-	this->api_mver_map[Kafka_FindCoordinator] = 2;
-	this->api_mver_map[Kafka_JoinGroup] = 5;
-	this->api_mver_map[Kafka_SyncGroup] = 3;
-	this->api_mver_map[Kafka_Heartbeat] = 3;
-	this->api_mver_map[Kafka_OffsetFetch] = 1;
-	this->api_mver_map[Kafka_OffsetCommit] = 7;
-	this->api_mver_map[Kafka_ListOffsets] = 1;
-	this->api_mver_map[Kafka_LeaveGroup] = 1;
-	this->api_mver_map[Kafka_ApiVersions] = 0;
-	this->api_mver_map[Kafka_SaslHandshake] = 1;
-	this->api_mver_map[Kafka_SaslAuthenticate] = 0;
-	this->api_mver_map[Kafka_DescribeGroups] = 0;
 }
 
 int KafkaRequest::encode_produce(struct iovec vectors[], int max)
@@ -2588,10 +2611,7 @@ int KafkaRequest::encode_syncgroup(struct iovec vectors[], int max)
 		}
 	}
 	else
-	{
 		append_i32(this->msgbuf, 0);
-		append_bytes(this->msgbuf, "");
-	}
 
 	this->cur_size = this->msgbuf.size();
 
@@ -3021,14 +3041,8 @@ static int kafka_meta_parse_topic(void **buf, size_t *size,
 {
 	KafkaMetaList lst;
 	int32_t topic_cnt;
+
 	CHECK_RET(parse_i32(buf, size, &topic_cnt));
-
-	if (topic_cnt < 0)
-	{
-		errno = EBADMSG;
-		return -1;
-	}
-
 	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		int16_t error;
@@ -3123,17 +3137,9 @@ int KafkaResponse::parse_produce(void **buf, size_t *size)
 	int32_t partition;
 	int64_t base_offset, log_append_time, log_start_offset;
 	int32_t throttle_time;
-
-	CHECK_RET(parse_i32(buf, size, &topic_cnt));
-
-	if (topic_cnt < 0)
-	{
-		errno = EBADMSG;
-		return -1;
-	}
-
 	int produce_timeout = this->config.get_produce_timeout() * 2;
 
+	CHECK_RET(parse_i32(buf, size, &topic_cnt));
 	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
@@ -3197,11 +3203,12 @@ int KafkaResponse::parse_produce(void **buf, size_t *size)
 
 int KafkaResponse::parse_fetch(void **buf, size_t *size)
 {
+	int32_t throttle_time;
+
 	this->toppar_list.rewind();
 	KafkaToppar *toppar;
 	while ((toppar = this->toppar_list.get_next()) != NULL)
 		toppar->clear_records();
-	int32_t throttle_time;
 
 	if (this->api_version >= 1)
 		CHECK_RET(parse_i32(buf, size, &throttle_time));
@@ -3215,14 +3222,6 @@ int KafkaResponse::parse_fetch(void **buf, size_t *size)
 	}
 
 	int32_t topic_cnt;
-	CHECK_RET(parse_i32(buf, size, &topic_cnt));
-
-	if (topic_cnt < 0)
-	{
-		errno = EBADMSG;
-		return -1;
-	}
-
 	std::string topic_name;
 	int32_t partition_cnt;
 	int32_t partition;
@@ -3231,6 +3230,7 @@ int KafkaResponse::parse_fetch(void **buf, size_t *size)
 	int64_t producer_id, first_offset;
 	int64_t high_watermark;
 
+	CHECK_RET(parse_i32(buf, size, &topic_cnt));
 	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
@@ -3289,25 +3289,18 @@ int KafkaResponse::parse_fetch(void **buf, size_t *size)
 
 int KafkaResponse::parse_listoffset(void **buf, size_t *size)
 {
-	int32_t throttle_time, topic_cnt;
-
-	if (this->api_version >= 2)
-		CHECK_RET(parse_i32(buf, size, &throttle_time));
-
-	CHECK_RET(parse_i32(buf, size, &topic_cnt));
-
-	if (topic_cnt < 0)
-	{
-		errno = EBADMSG;
-		return -1;
-	}
-
+	int32_t throttle_time;
+	int32_t topic_cnt;
 	std::string topic_name;
 	int32_t partition_cnt;
 	int32_t partition;
 	int64_t offset_timestamp, offset;
 	int32_t offset_cnt;
 
+	if (this->api_version >= 2)
+		CHECK_RET(parse_i32(buf, size, &throttle_time));
+
+	CHECK_RET(parse_i32(buf, size, &topic_cnt));
 	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
@@ -3510,21 +3503,13 @@ int KafkaMessage::kafka_parse_member_assignment(const char *bbuf, size_t n,
 {
 	void **buf = (void **)&bbuf;
 	size_t *size = &n;
-	int32_t topic_cnt, partition_cnt;
+	int32_t topic_cnt;
+	int32_t partition_cnt;
 	int16_t version;
 	struct list_head *pos, *tmp;
 	std::string topic_name;
 	int32_t partition;
 
-	CHECK_RET(parse_i16(buf, size, &version));
-	CHECK_RET(parse_i32(buf, size, &topic_cnt));
-
-	if (topic_cnt < 0)
-	{
-		errno = EBADMSG;
-		return -1;
-	}
-
 	list_for_each_safe(pos, tmp, cgroup->get_assigned_toppar_list())
 	{
 		KafkaToppar *toppar = list_entry(pos, KafkaToppar, list);
@@ -3532,6 +3517,8 @@ int KafkaMessage::kafka_parse_member_assignment(const char *bbuf, size_t n,
 		delete toppar;
 	}
 
+	CHECK_RET(parse_i16(buf, size, &version));
+	CHECK_RET(parse_i32(buf, size, &topic_cnt));
 	for (int32_t i = 0; i < topic_cnt; ++i)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
@@ -3599,13 +3586,6 @@ int KafkaResponse::parse_offsetfetch(void **buf, size_t *size)
 	int32_t partition;
 
 	CHECK_RET(parse_i32(buf, size, &topic_cnt));
-
-	if (topic_cnt < 0)
-	{
-		errno = EBADMSG;
-		return -1;
-	}
-
 	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
@@ -3642,17 +3622,10 @@ int KafkaResponse::parse_offsetcommit(void **buf, size_t *size)
 	int32_t partition_cnt;
 	int32_t partition;
 
-	if (this->api_version >= 1)
+	if (this->api_version >= 3)
 		CHECK_RET(parse_i32(buf, size, &throttle_time));
 
 	CHECK_RET(parse_i32(buf, size, &topic_cnt));
-
-	if (topic_cnt < 0)
-	{
-		errno = EBADMSG;
-		return -1;
-	}
-
 	for (int32_t topic_idx = 0; topic_idx < topic_cnt; ++topic_idx)
 	{
 		CHECK_RET(parse_string(buf, size, topic_name));
@@ -3819,7 +3792,7 @@ int KafkaResponse::handle_sasl_continue()
 		if (ret != (int)iovecs[i].iov_len)
 		{
 			if (ret >= 0)
-				errno = EAGAIN;
+				errno = ENOBUFS;
 			return -1;
 		}
 	}
